:root {
  font-family: 'Inter', system-ui, -apple-system, BlinkMacSystemFont, 'Segoe UI', sans-serif;
  color-scheme: light dark;
  color: #1a1a1a;
  background-color: #f2f4f8;
  line-height: 1.5;
  font-weight: 400;
}

body {
  margin: 0;
  min-height: 100vh;
  background: linear-gradient(180deg, #f5f7fb 0%, #eef2f7 100%);
}

* {
  box-sizing: border-box;
}

button {
  font-family: inherit;
}
